@@ -266,20 +266,18 @@ class RelationalGraphNeuralNetwork(nn.Module):
         """
         return self._dummy.device
 
-    def forward(self, x: list[tuple]) -> ForwardState:  # type: ignore
-        """Perform a forward pass through the R-GNN.
+    def _internal_forward(self, input: 'EncodedTensors') -> ForwardState:
+        """Run the neural network computation phase of the forward pass.
+
+        This private method contains the shared logic for running the MPNN forward pass
+        and creating the final ForwardState with readout functions.
 
         Args:
-            x: List of input tuples, where each tuple contains the inputs specified
-               by the input_spec (e.g., state, goal, actions).
+            input: The encoded tensors from the input encoders.
 
         Returns:
-            ForwardState object containing the final layer index and readout functions
-            for extracting outputs.
+            ForwardState object containing the final layer index and readout functions.
         """
-        # Create input using encoder-based specification
-        assert isinstance(x, list), 'Expected input to be a list.'
-        input = get_input_from_encoders(x, self._input_spec, self.get_device())
         # Pass the input through the MPNN module
         if len(self._hooks) > 0:
             def hook_function(layer_index: 'int', node_embeddings: 'torch.Tensor') -> 'None':
@@ -298,6 +296,45 @@ class RelationalGraphNeuralNetwork(nn.Module):
         curried_readouts = { name: make_readout_func(readout) for name, readout in self._readouts.items() }
         return ForwardState(self._hparam_config.num_layers - 1, curried_readouts)
 
+    def forward(self, x: list[tuple]) -> ForwardState:  # type: ignore
+        """Perform a forward pass through the R-GNN.
+
+        Args:
+            x: List of input tuples, where each tuple contains the inputs specified
+               by the input_spec (e.g., state, goal, actions).
+
+        Returns:
+            ForwardState object containing the final layer index and readout functions
+            for extracting outputs.
+        """
+        assert isinstance(x, list), 'Expected input to be a list.'
+        input = get_input_from_encoders(x, self._input_spec, self.get_device())
+        return self._internal_forward(input)
+
+    def curry_forward(self, x: list[tuple]) -> Callable[[], ForwardState]:
+        """Return a lambda that performs a forward pass through the R-GNN.
+
+        This method precomputes the encoded tensors but delays the MPNN computation
+        until the returned lambda is called. This is equivalent to model.forward(x)
+        but allows for separating the encoding phase from the neural network computation.
+
+        Args:
+            x: List of input tuples, where each tuple contains the inputs specified
+               by the input_spec (e.g., state, goal, actions).
+
+        Returns:
+            A lambda function that when called returns a ForwardState object containing
+            the final layer index and readout functions for extracting outputs.
+        """
+        assert isinstance(x, list), 'Expected input to be a list.'
+        input = get_input_from_encoders(x, self._input_spec, self.get_device())
+
+        # Define a curried function that captures the input and runs the internal forward pass.
+        def curried_forward() -> ForwardState:
+            return self._internal_forward(input)
+
+        return curried_forward
+
     def clone(self) -> 'RelationalGraphNeuralNetwork':
         """Create a deep copy of the model with identical weights and configuration.
 
